import importlib
import math
import random
import sys
import warnings
from datetime import date, time
from datetime import datetime
from pathlib import Path
from typing import Union, Dict, List, Generator, TypeVar, Type

from otlmow_model.OtlmowModel.BaseClasses.DateField import DateField
from otlmow_model.OtlmowModel.BaseClasses.DateTimeField import DateTimeField
from otlmow_model.OtlmowModel.BaseClasses.KeuzelijstField import KeuzelijstField
from otlmow_model.OtlmowModel.BaseClasses.OTLField import OTLField
from otlmow_model.OtlmowModel.BaseClasses.TimeField import TimeField
from otlmow_model.OtlmowModel.BaseClasses.URIField import URIField
from otlmow_model.OtlmowModel.BaseClasses.UnionTypeField import UnionTypeField
from otlmow_model.OtlmowModel.BaseClasses.UnionWaarden import UnionWaarden
from otlmow_model.OtlmowModel.Exceptions.AttributeDeprecationWarning import AttributeDeprecationWarning
from otlmow_model.OtlmowModel.Exceptions.CanNotClearAttributeError import CanNotClearAttributeError
from otlmow_model.OtlmowModel.Exceptions.ClassDeprecationWarning import ClassDeprecationWarning
from otlmow_model.OtlmowModel.Exceptions.CouldNotCreateInstanceError import CouldNotCreateInstanceError
from otlmow_model.OtlmowModel.Exceptions.MethodNotApplicableError import MethodNotApplicableError
from otlmow_model.OtlmowModel.Exceptions.NonStandardAttributeWarning import NonStandardAttributeWarning
from otlmow_model.OtlmowModel.Helpers.GenericHelper import get_titlecase_from_ns, get_ns_and_name_from_uri
from otlmow_model.OtlmowModel.Helpers.generated_lists import get_hardcoded_class_dict


class OTLAttribuut:
    def __init__(self, naam: str = '', label: str = '', objectUri: str = '', definition: str = '',
                 constraints: str = '', usagenote: str = '', deprecated_version: str = '', kardinaliteit_min: str = '1',
                 kardinaliteit_max: str = '1', field: Type[OTLField] = OTLField, readonly: bool = False,
                 readonlyValue=None, owner=None):
        super().__init__()
        self.naam: str = naam
        self.label: str = label
        self.objectUri: str = objectUri
        self.definition: str = definition
        self.constraints: str = constraints
        self.usagenote: str = usagenote
        self.deprecated_version: str = deprecated_version
        self.readonly: bool = readonly
        self.kardinaliteit_min: str = kardinaliteit_min
        self.kardinaliteit_max: str = kardinaliteit_max
        self._dotnotation: str  = ''
        self.owner = owner
        self.readonlyValue = None
        self.mark_to_be_cleared: bool = False
        self.waarde = None
        self.field: Type[OTLField] = field

        if self.field.waardeObject:
            def add_empty_value():
                prev_value = self.waarde
                if kardinaliteit_max == '1':
                    if prev_value is None:
                        new_value_object = self.field.waardeObject()
                        new_value_object._parent = self
                        self.set_waarde(new_value_object)
                    else:
                        raise RuntimeError("This attribute does not have a cardinality other than 1, therefore you can "
                                           "only call this method once per instance")
                else:
                    if prev_value is None:
                        prev_value = []
                    new_value_object = self.field.waardeObject()
                    new_value_object._parent = self
                    prev_value.append(new_value_object)
                    self.set_waarde(prev_value)

            self.add_empty_value = add_empty_value

        if kardinaliteit_max != '1':
            def add_value(value):
                l = self.waarde
                if self.waarde is None:
                    l = []
                l.append(value)
                self.set_waarde(l)

            self.add_value = add_value

        if readonly:
            self.__dict__["waarde"] = readonlyValue

    def add_value(self, value):
        raise MethodNotApplicableError(
            "This attribute does not have a cardinality other than 1 so simply assign your value directly instead of "
            "using this method")

    def get_waarde(self):
        if self.field.waardeObject and self.waarde is None:
            self.add_empty_value()
        return self.waarde

    def add_empty_value(self):
        """Helper method for datatypes UnionType, ComplexType, KwantWrd and Dte to add the underlying waarde object"""
        if not self.field.waardeObject:
            raise MethodNotApplicableError(
                "In order to use this method this object must be one of these types: UnionType, ComplexType, KwantWrd, "
                "Dte")

    def clear_value(self) -> None:
        if self.readonly:
            raise ValueError(f'attribute {self.naam} is readonly')
        if self.objectUri in {
            'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#AIMDBStatus.isActief',
            'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#AIMNaamObject.naam',
            'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#AIMToestand.toestand'
        } or self.field.objectUri == 'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#DtcIdentificator':
            raise CanNotClearAttributeError(f'attribute {self.naam} can not be cleared')
        if (self.owner is not None and hasattr(self.owner, '_parent') and self.owner._parent.field.objectUri ==
                'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#DtcIdentificator'):
            raise CanNotClearAttributeError(f'attribute {self.naam} can not be cleared')

        # complex takes precedence over cardinality
        # => complex + kard means clearing the value of each sub attribute of each item in the list
        # => complex means clearing the value of each sub attribute
        # => list means clearing the value

        if self.field.waardeObject is not None:
            if self.waarde is None:
                self.add_empty_value()
            if self.kardinaliteit_max != '1':
                for item in self.waarde:
                    for sub_attr in item:
                        if not sub_attr.readonly:
                            sub_attr.clear_value()
            else:
                for sub_attr in self.waarde:
                    if not sub_attr.readonly:
                        sub_attr.clear_value()
        else:
            self.waarde = None
            self.mark_to_be_cleared = True

    def _perform_cardinality_check(self, owner, value, kardinaliteit_max):
        kardinaliteit_min = int(self.kardinaliteit_min)
        if not isinstance(value, list):
            raise TypeError(f'expecting a list in {owner.__class__.__name__}.{self.naam}')
        elif isinstance(value, set):
            raise TypeError(f'expecting a non set type of list in {owner.__class__.__name__}.{self.naam}')
        elif 0 < len(value) < kardinaliteit_min:
            raise ValueError(
                f'expecting at least {kardinaliteit_min} element(s) in {owner.__class__.__name__}.{self.naam}')
        elif len(value) > kardinaliteit_max:
            raise ValueError(
                f'expecting at most {kardinaliteit_max} element(s) in {owner.__class__.__name__}.{self.naam}')

    def set_waarde(self, value, owner=None):
        self._perform_deprecation_check(self)

        if value is None:
            self.waarde = None
            return

        if value == self.field.clearing_value or (self.kardinaliteit_max != '1' and value == '88888888'):
            self.clear_value()
            return

        if self.kardinaliteit_max != '1':
            if self.kardinaliteit_max == '*':
                kardinaliteit_max = math.inf
            else:
                kardinaliteit_max = int(self.kardinaliteit_max)
            self._perform_cardinality_check(owner, value, kardinaliteit_max)
            converted_values = []
            for el_value in value:
                converted_value = self.field.convert_to_correct_type(el_value)
                if issubclass(self.field, KeuzelijstField):
                    converted_value = self.field.convert_to_invulwaarde(converted_value, self.field)

                field_validated = self.field.validate(converted_value, self)
                if not field_validated:
                    raise ValueError(
                        f'invalid value in list for {owner.__class__.__name__}.{self.naam}: {el_value} is not '
                        f'valid, must be valid for {self.field.naam}')
                converted_values.append(converted_value)
            self.waarde = converted_values
        else:
            if self.field.waardeObject is not None and isinstance(value, self.field.waardeObject):
                self.waarde = value
            else:
                converted_value = self.field.convert_to_correct_type(value)
                if issubclass(self.field, KeuzelijstField):
                    converted_value = self.field.convert_to_invulwaarde(converted_value, self.field)
                if self.field.validate(value=converted_value, attribuut=self):
                    self.waarde = converted_value
                else:
                    raise ValueError(
                        f'Could not assign the best effort converted value to {owner.__class__.__name__}.{self.naam}')

        # check if kwant Wrd inside a union type, if so, call clear_props
        if (owner is not None and value is not None and hasattr(owner, 'field') and owner.field.waardeObject is not None
                and (owner.field.waarde_shortcut_applicable and not isinstance(owner.field, UnionTypeField)
                     and owner.owner is not None and isinstance(owner.owner, UnionWaarden))):
            owner.owner.clear_other_props(f'_{owner.naam}')

    @staticmethod
    def _perform_deprecation_check(owner):
        if owner is not None:
            if owner.naam == 'waarde':
                owner = owner.owner._parent

            if hasattr(owner, 'deprecated_version') and owner.deprecated_version != '':
                if hasattr(owner, 'objectUri'):
                    warnings.warn(
                        message=f'{owner.objectUri} is deprecated since version {owner.deprecated_version}',
                        category=AttributeDeprecationWarning)
                elif hasattr(owner, 'typeURI'):
                    warnings.warn(message=f'{owner.typeURI} is deprecated since version {owner.deprecated_version}',
                                  category=AttributeDeprecationWarning)
                else:
                    warnings.warn(
                        message=f'used a class that is deprecated since version {owner.deprecated_version}',
                        category=AttributeDeprecationWarning)

    def __str__(self):
        return (f'information about {self.naam}:\n'
                f'naam: {self.naam}\n'
                f'uri: {self.objectUri}\n'
                f'definition: {self.definition}\n'
                f'label: {self.label}\n'
                f'usagenote: {self.usagenote}\n'
                f'constraints: {self.constraints}\n'
                f'readonly: {self.readonly}\n'
                f'kardinaliteit_min: {self.kardinaliteit_min}\n'
                f'kardinaliteit_max: {self.kardinaliteit_max}\n'
                f'deprecated_version: {self.deprecated_version}\n')

    def fill_with_dummy_data(self):
        if self.readonly:
            return

        if self.field.waardeObject is None:
            if self.naam == 'geometry':
                first_geom_type = self.owner._geometry_types[0]
                if first_geom_type == 'POINT Z':
                    self.set_waarde('POINT Z (200000 200000 0)')
                elif first_geom_type == 'LINESTRING Z':
                    self.set_waarde('LINESTRING Z (200000 200000 0, 200001 200001 1)')
                elif first_geom_type == 'POLYGON Z':
                    self.set_waarde('POLYGON Z ((200000 200000 0, 200001 200001 1, 200002 200002 2, 200000 200000 0))')
            else:
                if self.objectUri == 'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#NaampadObject.naampad':
                    naam_attr = get_attribute_by_name(self.owner, 'naam')
                    if naam_attr is not None and naam_attr.waarde is not None:
                        data = f'dummy/{naam_attr.waarde}'
                    else:
                        data = 'dummy/dummy'
                else:
                    data = self.field.create_dummy_data()
                if data is None or self.kardinaliteit_max == '1':
                    self.set_waarde(data)
                else:
                    self.set_waarde([data])
            return
        new_value_object = self.field.waardeObject()
        new_value_object._parent = self

        if isinstance(new_value_object, UnionWaarden):
            selected_attr = random.choice(list(new_value_object))
            selected_attr.fill_with_dummy_data()
        else:
            for a in new_value_object:
                a.fill_with_dummy_data()

        if self.kardinaliteit_max != '1':
            self.set_waarde([new_value_object])
        else:
            self.set_waarde(new_value_object)


T = TypeVar('T', bound='OTLObject')


class OTLObject(object):
    typeURI: str = None
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __setattr__(self, name, value):
        if name != 'typeURI':
            super(OTLObject, self).__setattr__(name, value)
        else:
            if hasattr(self, 'typeURI') and (value is not None or self.typeURI is not None):
                raise ValueError("The typeURI is an OSLOAttribute that indicates the class of the instance. "
                                 "Within a class this value is predefined and cannot be changed.")
            if URIField.validate(value, OTLAttribuut(naam='typeURI')):
                self.__dict__['value'] = value
            else:
                raise ValueError(f'{value} is not a valid value for typeURI.')

    def __init__(self):
        super().__init__()

        if hasattr(self, 'deprecated_version') and self.deprecated_version is not None:
            try:
                warnings.warn(message=f'{self.typeURI} is deprecated since version {self.deprecated_version}',
                              category=ClassDeprecationWarning)
            except KeyError:
                warnings.warn(
                    message=f'used a class ({self.__class__.__name__}) that is deprecated since version {self.deprecated_version}',
                    category=ClassDeprecationWarning)

    def clear_value(self, attribute_name: str) -> None:
        if attribute_name is None:
            raise ValueError('attribute_name is None')
        attr = get_attribute_by_name(self, attribute_name)
        if attr is None:
            raise ValueError(f'attribute {attribute_name} does not exist')
        attr.clear_value()

    def create_dict_from_asset(self, waarde_shortcut: bool = False, rdf: bool = False, cast_datetime: bool = False,
                               warn_for_non_otl_conform_attributes: bool = True,
                               allow_non_otl_conform_attributes: bool = True, ) -> Dict:
        """Converts this asset into a dictionary representation. This is now deprecated, use to_dict instead.

        :param waarde_shortcut: whether to use the waarde shortcut when processing the dictionary, defaults to False
        :type: bool
        :param rdf: whether to generate a dictionary where the keys are the URI's of the attributes
        rather than the names, defaults to False
        :type: bool
        :param cast_datetime: whether to convert dates, times and datetimes to strings, defaults to False
        :type: bool
        :param allow_non_otl_conform_attributes: whether to allow non OTL conform attributes. Raising ValueError if not, Defaults to True
        :type: bool
        :param warn_for_non_otl_conform_attributes: whether to generate warnings when using non OTL conform attributes. Defaults to True
        :type: bool

        :return: returns a dictionary representation of this asset
        :rtype: dict"""
        warnings.warn(message='create_dict_from_asset is deprecated, use to_dict instead',
                      category=AttributeDeprecationWarning)
        return create_dict_from_asset(
            otl_object=self, waarde_shortcut=waarde_shortcut, rdf=rdf, cast_datetime=cast_datetime,
            warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes,
            allow_non_otl_conform_attributes=allow_non_otl_conform_attributes)

    def fill_with_dummy_data(self):
        for attr in self:
            if attr is not None:
                attr.fill_with_dummy_data()

    def __repr__(self):
        return build_string_version(asset=self)

    def __iter__(self) -> Generator[OTLAttribuut, None, None]:
        yield from sorted(filter(lambda v: isinstance(v, OTLAttribuut), (vars(self).values())), key=lambda x: x.naam)

    def __eq__(self, other):
        return (create_dict_from_asset(self, warn_for_non_otl_conform_attributes=False) ==
                create_dict_from_asset(other, warn_for_non_otl_conform_attributes=False))

    def is_instance_of(self, otl_type: type, dynamic_created: bool = False, model_directory: Path = None):
        if dynamic_created:
            if model_directory is None:
                current_file_path = Path(__file__)
                model_directory = current_file_path.parent.parent.parent
            try:
                otl_type_typeURI = getattr(otl_type, 'typeURI')
            except AttributeError:
                return False
            dynamic_created_type = dynamic_create_type_from_uri(otl_type_typeURI, model_directory=model_directory)
            return isinstance(self, dynamic_created_type)

        check = isinstance(self, otl_type)
        if check:
            return True

        try:
            otl_type_typeURI = getattr(otl_type, 'typeURI')
        except AttributeError:
            return False

        t = type(self)
        if t.typeURI == otl_type_typeURI:
            return True
        return issubclass(self.__class__, otl_type)

    @classmethod
    def from_dict(cls: T, input_dict: Dict, model_directory: Path = None, rdf: bool = False,
                  cast_datetime: bool = False, waarde_shortcut: bool = False,
                  allow_non_otl_conform_attributes: bool = True, warn_for_non_otl_conform_attributes: bool = True
                  ) -> T:
        """Alternative constructor. Allows the instantiation of an object using a dictionary. Either start from the
        appropriate class or add a typeURI entry to the dictionary to get an instance of that type.

        :param input_dict: input dictionary, containing key value pairs for the attributes of the instance
        :type: dict
        :param model_directory: directory where the model is located, defaults to otlmow_model's own model
        :type: str
        :param rdf: whether to use uri's as keys instead of the names, defaults to False
        :type: bool
        :param waarde_shortcut: whether to use the waarde shortcut when processing the dictionary, defaults to False
        :type: bool
        :param cast_datetime: whether to convert dates, times and datetimes to strings, defaults to False
        :type: bool
        :param allow_non_otl_conform_attributes: whether to allow non OTL conform attributes. Raising ValueError if not, Defaults to True
        :type: bool
        :param warn_for_non_otl_conform_attributes: whether to generate warnings when using non OTL conform attributes. Defaults to True
        :type: bool
        :return: returns an instance where the values of the attributes matches the given dictionary
        :rtype: OTLObject"""
        if not rdf and 'typeURI' in input_dict:
            type_uri = input_dict['typeURI']
        elif rdf and 'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#AIMObject.typeURI' in input_dict:
            type_uri = input_dict['https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#AIMObject.typeURI']
        elif rdf and '@type' in input_dict:
            type_uri = input_dict['@type']
            del input_dict['@type']
        else:
            type_uri = cls.typeURI

        if type_uri is None:
            raise ValueError(
                'typeURI is None. Add a valid typeURI to the input dictionary or change the class you are using "from_dict" from.')

        if model_directory is None:
            current_file_path = Path(__file__)
            model_directory = current_file_path.parent.parent.parent

        try:
            o = dynamic_create_instance_from_uri(type_uri, model_directory=model_directory)
        except TypeError:
            raise ValueError(
                'typeURI is invalid. Add a valid typeURI to the input dictionary or change the class you are using "from_dict" from.')

        for k, v in input_dict.items():
            if k in {'typeURI', 'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#AIMObject.typeURI'}:
                continue
            set_value_by_dictitem(o, k, v, waarde_shortcut=waarde_shortcut, rdf=rdf,
                                  cast_datetime=cast_datetime,
                                  allow_non_otl_conform_attributes=allow_non_otl_conform_attributes,
                                  warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes)
        return o

    def to_dict(self, rdf: bool = False, cast_datetime: bool = False, waarde_shortcut: bool = False,
                allow_non_otl_conform_attributes: bool = True, warn_for_non_otl_conform_attributes: bool = True) -> Dict:
        """Converts this asset into a dictionary representation

        :param waarde_shortcut: whether to use the waarde shortcut when processing the dictionary, defaults to False
        :type: bool
        :param rdf: whether to generate a dictionary where the keys are the URI's of the attributes
        rather than the names, defaults to False
        :type: bool
        :param cast_datetime: whether to convert dates, times and datetimes to strings, defaults to False
        :type: bool
        :param allow_non_otl_conform_attributes: whether to allow non OTL conform attributes. Raising ValueError if not, Defaults to True
        :type: bool
        :param warn_for_non_otl_conform_attributes: whether to generate warnings when using non OTL conform attributes. Defaults to True
        :type: bool

        :return: returns a dictionary representation of this asset
        :rtype: dict"""
        return create_dict_from_asset(
            otl_object=self, waarde_shortcut=waarde_shortcut, rdf=rdf, cast_datetime=cast_datetime,
            warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes,
            allow_non_otl_conform_attributes=allow_non_otl_conform_attributes)


def create_dict_from_asset(otl_object: OTLObject, waarde_shortcut=False, rdf: bool = False,
                           cast_datetime: bool = False, allow_non_otl_conform_attributes: bool = True,
                           warn_for_non_otl_conform_attributes: bool = True) -> Dict:
    """Creates a dictionary from an OTLObject with key value pairs for attributes and their values. Saves the type of the object in typeURI (or @type for the RDF dict)

    :param otl_object: input object to be transformed
    :type: OTLObject
    :param waarde_shortcut: whether to use the waarde shortcut when processing the dictionary, defaults to False
    :type: bool
    :param rdf: whether to generate a dictionary where the keys are the URI's of the attributes rather than the names, defaults to False
    :type: bool
    :param cast_datetime: whether to convert dates, times and datetimes to strings, defaults to False
    :type: bool
    :param suppress_warnings_non_standardised_attributes: whether to suppress the warning that are raised because the object has attributes that aren't standardised, defaults to False
    :type: bool
    :param warn_for_non_otl_conform_attributes: whether to generate warnings when using non OTL conform attributes. Defaults to True
    :type: bool

    :return: returns an dictionary where the values of the dictionary matches the given input object
    :rtype: dict"""
    if rdf:
        d = _recursive_create_rdf_dict_from_asset(
            asset=otl_object, waarde_shortcut=waarde_shortcut, cast_datetime=cast_datetime,
            warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes,
            allow_non_otl_conform_attributes=allow_non_otl_conform_attributes)
    else:
        d = _recursive_create_dict_from_asset(
            asset=otl_object, waarde_shortcut=waarde_shortcut, cast_datetime=cast_datetime,
            warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes,
            allow_non_otl_conform_attributes=allow_non_otl_conform_attributes)

    if d is None:
        d = {}
    if rdf:
        d['@type'] = otl_object.typeURI
    else:
        d['typeURI'] = otl_object.typeURI
    return d


def _recursive_create_dict_from_asset(
        asset: Union[OTLObject, OTLAttribuut, list, dict], waarde_shortcut: bool = False,
        cast_datetime: bool = False, warn_for_non_otl_conform_attributes: bool = True,
        allow_non_otl_conform_attributes: bool = True
) -> Union[Dict, List[Dict]]:
    if isinstance(asset, list) and not isinstance(asset, dict):
        l = []
        for item in asset:
            dict_item = _recursive_create_dict_from_asset(
                asset=item, waarde_shortcut=waarde_shortcut, cast_datetime=cast_datetime,
                warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes,
                allow_non_otl_conform_attributes=allow_non_otl_conform_attributes)
            if dict_item is not None:
                l.append(dict_item)
        return l
    else:
        d = {}
        for attr_key, attr in vars(asset).items():
            if attr_key in {'_parent', '_valid_relations', '_geometry_types'}:
                continue
            if isinstance(attr, OTLAttribuut):
                if not attr.mark_to_be_cleared:
                    if attr.waarde is None:
                        continue
                    if attr.waarde == []:
                        d[attr.naam] = []
                        continue

                if attr.field.waardeObject is not None:  # complex
                    if waarde_shortcut and attr.field.waarde_shortcut_applicable:  # waarde shortcut
                        if isinstance(attr.waarde, list):
                            item_list = []
                            for item in attr.waarde:
                                if item._waarde.mark_to_be_cleared:
                                    item_list.append(item._waarde.field.clearing_value)
                                else:
                                    item_list.append(item._waarde.waarde)
                            if len(item_list) > 0:
                                d[attr.naam] = item_list
                        else:
                            if attr.waarde._waarde.mark_to_be_cleared:
                                dict_item = attr.waarde._waarde.field.clearing_value
                            else:
                                dict_item = attr.waarde.waarde
                            if dict_item is not None:
                                d[attr.naam] = dict_item
                    else:  # regular complex or union
                        dict_item = _recursive_create_dict_from_asset(
                            asset=attr.waarde, waarde_shortcut=waarde_shortcut, cast_datetime=cast_datetime,
                            warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes,
                            allow_non_otl_conform_attributes=allow_non_otl_conform_attributes)
                        if dict_item is not None:
                            d[attr.naam] = dict_item
                else:
                    if attr.mark_to_be_cleared:
                        d[attr.naam] = attr.field.clearing_value
                    elif cast_datetime:
                        if attr.field == TimeField:
                            if isinstance(attr.waarde, list):
                                d[attr.naam] = [time.strftime(list_item, "%H:%M:%S") for list_item in attr.waarde]
                            else:
                                d[attr.naam] = time.strftime(attr.waarde, "%H:%M:%S")
                        elif attr.field == DateField:
                            if isinstance(attr.waarde, list):
                                d[attr.naam] = [date.strftime(list_item, "%Y-%m-%d") for list_item in attr.waarde]
                            else:
                                d[attr.naam] = date.strftime(attr.waarde, "%Y-%m-%d")
                        elif attr.field == DateTimeField:
                            if isinstance(attr.waarde, list):
                                d[attr.naam] = [DateTimeField.value_default(list_item)
                                                for list_item in attr.waarde]
                            else:
                                d[attr.naam] = DateTimeField.value_default(attr.waarde)
                        else:
                            d[attr.naam] = attr.waarde
                    else:
                        d[attr.naam] = attr.waarde
            else:
                if not attr_key.startswith('_'):
                    if allow_non_otl_conform_attributes:
                        if warn_for_non_otl_conform_attributes:
                            warnings.warn(
                                message=f'{attr_key} is a non standardized attribute of {asset.__class__.__name__}. '
                                        f'The attribute will be added on the instance.', stacklevel=2,
                                category=NonStandardAttributeWarning)
                        d[attr_key] = attr
                    else:
                        raise ValueError(f'{attr_key} is a non standardized attribute of {asset.__class__.__name__}. '
                                         f'If you want to allow this, set allow_non_otl_conform_attributes to True.')
                else:
                    raise ValueError(
                        f'{attr_key} is a non standardized attribute of {asset.__class__.__name__}. '
                        f'While this is supported, the key can not start with "_".')

        if len(d.items()) > 0:
            return d


def _recursive_create_rdf_dict_from_asset(
        asset: Union[OTLObject, OTLAttribuut, list, dict], waarde_shortcut: bool = False,
        cast_datetime: bool = False, allow_non_otl_conform_attributes: bool = True,
        warn_for_non_otl_conform_attributes: bool = True) -> Union[Dict, List[Dict]]:
    if isinstance(asset, list) and not isinstance(asset, dict):
        l = []
        for item in asset:
            dict_item = _recursive_create_rdf_dict_from_asset(
                asset=item, waarde_shortcut=waarde_shortcut, cast_datetime=cast_datetime,
                warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes,
                allow_non_otl_conform_attributes=allow_non_otl_conform_attributes)
            if dict_item is not None:
                l.append(dict_item)
        if len(l) > 0:
            return l
    else:
        d = {}
        for attr_key, attr in vars(asset).items():
            if attr_key in {'_parent', '_valid_relations', '_geometry_types'}:
                continue
            if isinstance(attr, OTLAttribuut):
                if not attr.mark_to_be_cleared:
                    if attr.waarde is None:
                        continue
                    if attr.waarde == []:
                        d[attr.naam] = []
                        continue

                if attr.field.waardeObject is not None:  # complex
                    if waarde_shortcut and attr.field.waarde_shortcut_applicable:
                        if isinstance(attr.waarde, list):
                            item_list = []
                            for item in attr.waarde:
                                if item._waarde.mark_to_be_cleared:
                                    item_list.append(item._waarde.field.clearing_value)
                                else:
                                    item_list.append(item._waarde.waarde)
                            if len(item_list) > 0:
                                d[attr.objectUri] = item_list
                        else:
                            if attr.waarde._waarde.mark_to_be_cleared:
                                dict_item = attr.waarde._waarde.field.clearing_value
                            else:
                                dict_item = attr.waarde.waarde
                            if dict_item is not None:
                                d[attr.objectUri] = dict_item
                    else:
                        if attr.mark_to_be_cleared:
                            for a in attr.waarde:
                                a.mark_to_be_cleared = True

                        dict_item = _recursive_create_rdf_dict_from_asset(
                            asset=attr.waarde, waarde_shortcut=waarde_shortcut, cast_datetime=cast_datetime,
                            warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes,
                            allow_non_otl_conform_attributes=allow_non_otl_conform_attributes)
                        if dict_item is not None:
                            d[attr.objectUri] = dict_item
                else:
                    if attr.mark_to_be_cleared:
                        d[attr.objectUri] = attr.field.clearing_value
                    elif cast_datetime:
                        if attr.field == TimeField:
                            if isinstance(attr.waarde, list):
                                d[attr.objectUri] = [time.strftime(list_item, "%H:%M:%S") for list_item in attr.waarde]
                            else:
                                d[attr.objectUri] = time.strftime(attr.waarde, "%H:%M:%S")
                        elif attr.field == DateField:
                            if isinstance(attr.waarde, list):
                                d[attr.objectUri] = [date.strftime(list_item, "%Y-%m-%d") for list_item in attr.waarde]
                            else:
                                d[attr.objectUri] = date.strftime(attr.waarde, "%Y-%m-%d")
                        elif attr.field == DateTimeField:
                            if isinstance(attr.waarde, list):
                                d[attr.objectUri] = [datetime.strftime(list_item, "%Y-%m-%d %H:%M:%S")
                                                     for list_item in attr.waarde]
                            else:
                                d[attr.objectUri] = datetime.strftime(attr.waarde, "%Y-%m-%d %H:%M:%S")
                        elif issubclass(attr.field, KeuzelijstField):
                            if isinstance(attr.waarde, list):
                                if attr.waarde == [None]:
                                    d[attr.objectUri] = []
                                else:
                                    d[attr.objectUri] = [attr.field.options[list_item].objectUri for list_item in
                                                         attr.waarde]
                            else:
                                d[attr.objectUri] = attr.field.options[attr.waarde].objectUri
                        else:
                            d[attr.objectUri] = attr.waarde
                    elif issubclass(attr.field, KeuzelijstField):
                        if isinstance(attr.waarde, list):
                            if attr.waarde == [None]:
                                d[attr.objectUri] = []
                            else:
                                d[attr.objectUri] = [attr.field.options[list_item].objectUri for list_item in
                                                     attr.waarde]
                        else:
                            d[attr.objectUri] = attr.field.options[attr.waarde].objectUri
                    else:
                        d[attr.objectUri] = attr.waarde
            else:
                if not attr_key.startswith('_'):
                    if allow_non_otl_conform_attributes:
                        if warn_for_non_otl_conform_attributes:
                            warnings.warn(
                                message=f'{attr_key} is a non standardized attribute of {asset.__class__.__name__}. '
                                        f'The attribute will be added on the instance.', stacklevel=2,
                                category=NonStandardAttributeWarning)
                        d[attr_key] = attr
                    else:
                        raise ValueError(f'{attr_key} is a non standardized attribute of {asset.__class__.__name__}. '
                                         f'If you want to allow this, set allow_non_otl_conform_attributes to True.')
                else:
                    raise ValueError(
                        f'{attr_key} is a non standardized attribute of {asset.__class__.__name__}. '
                        f'While this is supported, the key can not start with "_".')

        if len(d.items()) > 0:
            return d


def clean_dict(d) -> Union[Dict, None]:
    """Recursively remove None values and empty dicts from input dict"""
    if d is None:
        return None
    for k in list(d):
        v = d[k]
        if isinstance(v, dict):
            clean_dict(v)
            if len(v.items()) == 0:
                del d[k]
        if v is None:
            del d[k]
    return d


def build_string_version(asset, indent: int = 4) -> str:
    indent = max(indent, 4)
    d = create_dict_from_asset(asset, warn_for_non_otl_conform_attributes=False)
    string_version = '\n'.join(_make_string_version_from_dict(d, level=1, indent=indent, prefix='    '))
    if string_version != '':
        string_version = '\n' + string_version
    return f'<{asset.__class__.__name__}> object\n{" " * indent}typeURI : {asset.typeURI}{string_version}'


def _make_string_version_from_dict(d, level: int = 0, indent: int = 4, list_index: int = -1, prefix: str = '') -> List:
    lines = []

    if list_index != -1:
        index_string = f'[{list_index}]'
        index_string += ' ' * (indent - len(index_string))
        prefix += index_string

    for key in sorted(d):
        if key == 'typeURI':
            continue
        value = d[key]
        if isinstance(value, float) and value == 88888888.0:
            value = '88888888 <value_marked_to_be_cleared>'
        elif str(value) == '88888888':
            value = '88888888 <value_marked_to_be_cleared>'

        if isinstance(value, dict):
            lines.append(f'{prefix}{key} :')
            lines.extend(_make_string_version_from_dict(value, level=level + 1, indent=indent,
                                                        prefix=prefix + ' ' * indent))
        elif isinstance(value, list):
            lines.append(f'{prefix}{key} :')
            for index, item in enumerate(value):
                if index == 10:
                    if len(value) == 11:
                        lines.append(f'{prefix}...(1 more item)')
                    else:
                        lines.append(f'{prefix}...({len(value) - 10} more items)')
                    break
                if isinstance(item, dict):
                    lines.extend(_make_string_version_from_dict(item, level=level, indent=indent, list_index=index,
                                                                prefix=prefix))
                else:
                    index_string = f'[{index}]'
                    index_string += ' ' * (indent - len(index_string))
                    lines.append(prefix + index_string + f'{item}')
        else:
            lines.append(f'{prefix}{key} : {value}')
    return lines


def get_attribute_by_uri(instance_or_attribute, key: str) -> Union[OTLAttribuut, None]:
    return next((v for v in instance_or_attribute if v.objectUri == key), None)


def get_attribute_by_name(instance_or_attribute, key: str) -> Union[OTLAttribuut, None]:
    return getattr(instance_or_attribute, f'_{key}', None)


# dict encoder = asset object to dict
# dict decoder = dict to asset object

def set_value_by_dictitem(instance_or_attribute: Union[OTLObject, OTLAttribuut], key: str, value,
                          waarde_shortcut: bool = False, rdf: bool = False, cast_datetime: bool = False,
                          allow_non_otl_conform_attributes: bool = True,
                          warn_for_non_otl_conform_attributes: bool = True):
    if instance_or_attribute is None:
        raise ValueError('instance_or_attribute cannot be None')
    if key is None or key == '':
        raise ValueError('key cannot be empty')

    if rdf:
        attribute_to_set = get_attribute_by_uri(instance_or_attribute, key)
    else:
        attribute_to_set = get_attribute_by_name(instance_or_attribute, key)

    if attribute_to_set is None:
        if allow_non_otl_conform_attributes:
            if warn_for_non_otl_conform_attributes:
                warnings.warn(
                    message=f'Attribute with name "{key}" can not be found on the given instance or attribute. '
                            'Assuming this is a non standardized attribute. Setting the attribute with setattr',
                    category=NonStandardAttributeWarning)
            setattr(instance_or_attribute, key, value)
            return
        else:
            raise ValueError(f'Attribute with name "{key}" can not be found on the given instance or attribute. '
                             'Assuming this is a non standardized attribute. If you want to allow this, set '
                             'allow_non_otl_conform_attributes to True.')

    if value == attribute_to_set.field.clearing_value or value == [attribute_to_set.field.clearing_value]:
        attribute_to_set.clear_value()
        return

    if attribute_to_set.field.waardeObject is not None:  # complex / union / KwantWrd / dte
        if isinstance(value, list):
            for index, list_item in enumerate(value):
                if attribute_to_set.waarde is None or len(attribute_to_set.waarde) <= index:
                    attribute_to_set.add_empty_value()

                if attribute_to_set.field.waarde_shortcut_applicable and waarde_shortcut:  # dte / kwantWrd
                    if list_item == attribute_to_set.waarde[index]._waarde.field.clearing_value:
                        attribute_to_set.waarde[index]._waarde.clear_value()
                        continue
                    attribute_to_set.waarde[index]._waarde.set_waarde(list_item)

                else:  # complex / union
                    for k, v in list_item.items():
                        set_value_by_dictitem(attribute_to_set.waarde[index], k, v, waarde_shortcut, rdf=rdf,
                                              cast_datetime=cast_datetime,
                                              allow_non_otl_conform_attributes=allow_non_otl_conform_attributes,
                                              warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes)

        elif isinstance(value, dict):  # only complex / union possible
            if attribute_to_set.waarde is None:
                attribute_to_set.add_empty_value()

            if attribute_to_set.kardinaliteit_max != '1':
                for k, v in value.items():
                    set_value_by_dictitem(attribute_to_set.waarde[0], k, v, waarde_shortcut, rdf=rdf,
                                          cast_datetime=cast_datetime,
                                          allow_non_otl_conform_attributes=allow_non_otl_conform_attributes,
                                          warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes)
            else:
                for k, v in value.items():
                    set_value_by_dictitem(attribute_to_set.waarde, k, v, waarde_shortcut, rdf=rdf,
                                          cast_datetime=cast_datetime,
                                          allow_non_otl_conform_attributes=allow_non_otl_conform_attributes,
                                          warn_for_non_otl_conform_attributes=warn_for_non_otl_conform_attributes)
        else:  # must be a dte / kwantWrd
            if attribute_to_set.waarde is None:
                attribute_to_set.add_empty_value()

            if cast_datetime and attribute_to_set.waarde._waarde.field in [TimeField, DateField, DateTimeField]:
                value = attribute_to_set.waarde._waarde.field.convert_to_correct_type(value=value, log_warnings=False)
            attribute_to_set.waarde._waarde.set_waarde(value)
    else:
        if cast_datetime and attribute_to_set.field in [TimeField, DateField, DateTimeField]:
            value = attribute_to_set.field.convert_to_correct_type(value=value, log_warnings=False)
        attribute_to_set.set_waarde(value)


def dynamic_create_type_from_ns_and_name(namespace: str, class_name: str, model_directory: Path = None) -> type:
    """Loads the OTL class module and attempts to return the type using the name and namespace of the class

    :param namespace: namespace of the class
    :type: str
    :param class_name: class name to instantiate
    :type: str
    :param model_directory: directory where the model is located, defaults to otlmow_model's own model
    :type: str
    :return: returns an instance of class_name in the given namespace, located from directory
    :rtype: type
    """
    try:
        class_dict = get_hardcoded_class_dict(model_directory=model_directory)
    except FileNotFoundError as e:
        raise CouldNotCreateInstanceError(
            e.message + '\nMake sure you are directing to the (parent) directory '
            'where OtlmowModel is located in.'
        ) from e
    class_entry = next((c for c in class_dict.values() if c['ns'] == namespace and c['name'] == class_name), None)
    if class_entry is not None:
        return _create_type_from_class_dict_entry(class_entry, model_directory=model_directory)

    error = CouldNotCreateInstanceError(
        f'The combination of the namespace {namespace} and the class name {class_name} is not found in the hardcoded '
        f'class dictionary.')

    ns_match = next((c for c in class_dict.values() if c['ns'] == namespace), None)
    if ns_match is None:
        error.message += f'\nNo classes found with the namespace {namespace}.'
        from difflib import get_close_matches
        closest_matches = get_close_matches(namespace, {c['ns'] for c in class_dict.values()}, n=5, cutoff=0.8)
        closest_matches_string = '", "'.join(closest_matches)
        error.message += f'\nDid you mean one of these namespaces? "{closest_matches_string}"'
        raise error

    from difflib import get_close_matches
    closest_matches = get_close_matches(class_name, {c['name'] for c in class_dict.values()}, n=5, cutoff=0.8)
    closest_matches_string = '", "'.join(closest_matches)

    error.message += (f'\nNo classes found with the namespace {namespace}.'
                      f'nDid you mean one of these class names? "{closest_matches_string}"')
    error.closest_matches = closest_matches
    raise error





def dynamic_create_instance_from_ns_and_name(namespace: str, class_name: str, model_directory: Path = None) -> OTLObject:
    """Loads the OTL class module and attempts to instantiate the class using the name and namespace of the class.
    Caches imported modules for performance improvement.

    :param namespace: namespace of the class
    :type: str
    :param class_name: class name to instantiate
    :type: str
    :param model_directory: directory where the model is located, defaults to otlmow_model's own model
    :type: Path
    :return: returns an instance of class_name in the given namespace, located from directory
    :rtype: OTLObject
    """
    type_ = dynamic_create_type_from_ns_and_name(namespace=namespace, class_name=class_name,
                                                 model_directory=model_directory)
    return type_()


def dynamic_create_instance_from_uri(class_uri: str, model_directory: Path = None) -> OTLObject:
    """Loads the OTL class module and attempts to instantiate the class using the URI of the class.

    :param class_uri: URI of the class
    :type: str
    :param model_directory: directory where the model is located, defaults to otlmow_model's own model
    :type: Path
    :return: returns an instance of the class with the given URI, located from directory
    :rtype: OTLObject
    """
    type_ = dynamic_create_type_from_uri(class_uri, model_directory=model_directory)
    return type_()


def dynamic_create_type_from_uri(class_uri: str, model_directory: Path = None) -> type:
    """Loads the OTL class module and attempts to return the type using the URI of the class.

    :param class_uri: URI of the class
    :type: str
    :param model_directory: directory where the model is located, defaults to otlmow_model's own model
    :type: Path
    :return: returns a type object created from the class URI
    :rtype: type
    """
    try:
        class_dict = get_hardcoded_class_dict(model_directory=model_directory)
    except FileNotFoundError as e:
        raise CouldNotCreateInstanceError(
            e.message + '\nMake sure you are directing to the (parent) directory '
            'where OtlmowModel is located in.'
        ) from e
    if class_uri in class_dict:
        return _create_type_from_class_dict_entry(class_dict[class_uri], model_directory=model_directory)

    from difflib import get_close_matches
    closest_matches = get_close_matches(class_uri, class_dict.keys(), n=5, cutoff=0.8)
    closest_matches_string = '", "'.join(closest_matches)

    error = CouldNotCreateInstanceError(
        f'Class URI {class_uri} is not found in the hardcoded class dictionary. '
        f'Did you mean one of these? "{closest_matches_string}"')
    error.closest_matches = closest_matches
    raise error


_imported_modules = {}

def _create_type_from_class_dict_entry(class_dict_entry: Dict[str, str], model_directory: Path = None) -> type:
    """Creates a type from a class dictionary entry.

    :param class_dict_entry: dictionary containing the class information
    :type class_dict_entry: dict
    :param model_directory: directory where the model is located, defaults to otlmow_model's own model
    :type model_directory: Path
    :return: returns a type object created from the class dictionary entry
    :rtype: type
    """
    global _imported_modules
    namespace = class_dict_entry['ns']
    class_name = class_dict_entry['name']
    module_key = (namespace, class_name)

    if module_key in _imported_modules:
        mod = _imported_modules[module_key]
    else:
        model_directory_given = model_directory is not None
        if model_directory is None:
            current_file_path = Path(__file__)
            model_directory = current_file_path.parent.parent.parent

        namespace_path = '' if namespace == '' else f'{get_titlecase_from_ns(namespace)}.'
        module_path = f'OtlmowModel.Classes.{namespace_path}{class_name}'

        try:
            if str(model_directory) not in sys.path:
                sys.path.insert(1, str(model_directory))
            mod = importlib.import_module(module_path)
            _imported_modules[module_key] = mod
        except ModuleNotFoundError as e:
            error = CouldNotCreateInstanceError(
                f'When dynamically creating an object of class {class_name}, the import failed. ')
            if model_directory_given:
                error.message += 'Make sure you are directing to the (parent) directory where OtlmowModel is located in.'
            raise error from e

    return getattr(mod, class_name)



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.Abstracten.Proef import Proef
from otlmow_model.OtlmowModel.BaseClasses.FloatOrDecimalField import FloatOrDecimalField
from otlmow_model.OtlmowModel.GeometrieTypes.PuntGeometrie import PuntGeometrie
from otlmow_model.OtlmowModel.GeometrieTypes.LijnGeometrie import LijnGeometrie
from otlmow_model.OtlmowModel.GeometrieTypes.VlakGeometrie import VlakGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class ProefLuminantie(Proef, PuntGeometrie, LijnGeometrie, VlakGeometrie):
    """Bij gebrek aan Qd-meting kan de luminantiefactor ß van wegmarkeringen gebruikt worden om het contrast met het wegdek en bijgevolg de dagzichtbaarheid te bepalen."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/proefenmeting#ProefLuminantie'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IsInspectieVan', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Markering', direction='o')  # o = direction: outgoing

        self._luminantiefactor = OTLAttribuut(field=FloatOrDecimalField,
                                              naam='luminantiefactor',
                                              label='luminantiefactor',
                                              objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/proefenmeting#ProefLuminantie.luminantiefactor',
                                              definition='Waarde om het contrast met het wegdek en bijgevolg de dagzichtbaarheid te bepalen.',
                                              owner=self)

    @property
    def luminantiefactor(self) -> float:
        """Waarde om het contrast met het wegdek en bijgevolg de dagzichtbaarheid te bepalen."""
        return self._luminantiefactor.get_waarde()

    @luminantiefactor.setter
    def luminantiefactor(self, value):
        self._luminantiefactor.set_waarde(value, owner=self)



import datetime
import logging
import warnings
from datetime import timedelta
from random import randrange
from typing import Optional, Any

from otlmow_model.OtlmowModel.Exceptions.CouldNotConvertToCorrectTypeError import CouldNotConvertToCorrectTypeError
from otlmow_model.OtlmowModel.BaseClasses.OTLField import OTLField
from otlmow_model.OtlmowModel.warnings.IncorrectTypeWarning import IncorrectTypeWarning


class DateTimeField(OTLField):
    """Beschrijft een datum volgens http://www.w3.org/2001/XMLSchema#dateTime."""
    naam = 'DateTime'
    objectUri = 'http://www.w3.org/2001/XMLSchema#dateTime'
    definition = 'Beschrijft een datum volgens http://www.w3.org/2001/XMLSchema#dateTime.'
    label = 'Datumtijd'
    usagenote = 'https://www.w3.org/TR/xmlschema-2/#dateTime'
    clearing_value = '88888888'
    native_type = datetime.datetime

    @classmethod
    def validate(cls, value: Any, attribuut) -> bool:
        if value is not None and not isinstance(value, datetime.datetime):
            raise TypeError(f'expecting datetime in {attribuut.naam}')
        return True

    @classmethod
    def convert_to_correct_type(cls, value: Any, log_warnings: bool = True) -> Optional[datetime.datetime]:
        if value is None:
            return None
        if isinstance(value, bool):
            raise CouldNotConvertToCorrectTypeError(
                f'{value} could not be converted to correct type (implied by {cls.__name__})')
        if isinstance(value, datetime.datetime):
            return value
        if isinstance(value, datetime.date):
            if log_warnings:
                warnings.warn(category=IncorrectTypeWarning,
                              message='Assigned a date to a datetime datatype. '
                                      'Automatically converted to the correct type. Please change the type')
            return datetime.datetime(year=value.year, month=value.month, day=value.day)
        if isinstance(value, int):
            if log_warnings:
                warnings.warn(category=IncorrectTypeWarning,
                              message='Assigned a int to a datetime datatype. '
                                      'Automatically converted to the correct type. Please change the type')
            timestamp = datetime.datetime.fromtimestamp(value, datetime.timezone.utc)
            return datetime.datetime(timestamp.year, timestamp.month, timestamp.day, timestamp.hour, timestamp.minute,
                                     timestamp.second)
        if isinstance(value, str):
            try:
                if 'T' in value:
                    if '.' in value:
                        dt = datetime.datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")
                    else:
                        dt = datetime.datetime.strptime(value, "%Y-%m-%dT%H:%M:%S")
                else:
                    if '.' in value:
                        dt = datetime.datetime.strptime(value, "%Y-%m-%d %H:%M:%S.%f")
                    else:
                        dt = datetime.datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
                if log_warnings:
                    warnings.warn(category=IncorrectTypeWarning,
                                  message='Assigned a string to a datetime datatype. '
                                          'Automatically converted to the correct type. Please change the type')
                return dt
            except ValueError:
                try:
                    if 'T' in value:
                        dt = datetime.datetime.strptime(value, "%d/%m/%YT%H:%M:%S")
                    else:
                        dt = datetime.datetime.strptime(value, "%d/%m/%Y %H:%M:%S")
                    if log_warnings:
                        warnings.warn(category=IncorrectTypeWarning,
                                      message='Assigned a string to a datetime datatype. '
                                              'Automatically converted to the correct type. Please change the type')
                    return dt
                except Exception as e:
                    raise CouldNotConvertToCorrectTypeError(
                        f'{value} could not be converted to correct type (implied by {cls.__name__})'
                    ) from e
        try:
            return datetime.datetime(value)
        except Exception as exc:
            raise CouldNotConvertToCorrectTypeError(
                f'{value} could not be converted to correct type (implied by {cls.__name__})'
            ) from exc

    @classmethod
    def value_default(cls, value: datetime.datetime) -> Optional[str]:
        if value is None:
            return None
        if not isinstance(value, datetime.datetime):
            raise TypeError(f'Expecting datetime.datetime in {cls.__name__} and got {type(value)} when trying to '
                            f'parse {value}')
        if value.microsecond > 0:
            return value.strftime("%Y-%m-%d %H:%M:%S.%f")
        else:
            return value.strftime("%Y-%m-%d %H:%M:%S")

    def __str__(self) -> str:
        return OTLField.__str__(self)

    @staticmethod
    def random_date(start: datetime.datetime, end: datetime.datetime) -> datetime.datetime:
        delta = end - start
        int_delta = (delta.days * 24 * 60 * 60) + delta.seconds
        random_second = randrange(int_delta)
        return start + timedelta(seconds=random_second)

    @classmethod
    def create_dummy_data(cls) -> datetime.datetime:
        return DateTimeField.random_date(start=datetime.datetime(2000, 1, 1),
                                         end=datetime.datetime(2020, 1, 1))



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.ImplementatieElement.AIMNaamObject import AIMNaamObject
from ...Datatypes.KlVoorschakelapparaatType import KlVoorschakelapparaatType
from otlmow_model.OtlmowModel.GeometrieTypes.PuntGeometrie import PuntGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class Voorschakelapparaat(AIMNaamObject, PuntGeometrie):
    """Het voorschakelapparaat van een ontladingslamp in ruime zin omvat alle componenten die in serie of in parallel met de lamp geschakeld worden om haar goede werking te verzekeren. Ze bestaat in 2 uitvoeringsvormen elektromagnetische en elektronische. Voor de elektromagnetische bestaat deze uit *de ballast: een elektromagnetische eenheid, die d.m.v. passieve componenten zoals een spoel of een condensator en/of actieve componenten, hoofdzakelijk dient om de lampstroom te beperken tot de vereiste waarde; *een starter of een ontsteker: levert de vereiste ontsteekspanning; *een condensator:is een component die de faseverschuiving tussen de stroom en spanning tot een aanvaardbare waarde terugbrengt en op die manier de arbeidsfactor verbetert; *de eventuele externe beveiligingen; *de onderlinge bekabeling."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Voorschakelapparaat'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#VerlichtingstoestelNaHP', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Sturing', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Armatuurcontroller', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Sturing', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Montagekast', direction='u', deprecated='2.4.0')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Sturing', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Stroomkring', direction='u', deprecated='2.4.0')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Sturing', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#VerlichtingstoestelNaHP', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Sturing', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#WVOpvoertransformator', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Voedt', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Laagspanningsbord', direction='i')  # i = direction: incoming
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Voedt', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Stroomkring', direction='i', deprecated='2.4.0')  # i = direction: incoming
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Voedt', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#WVOpvoertransformator', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Voedt', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#WVOpvoertransformator', direction='i')  # i = direction: incoming
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#VoedtAangestuurd', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Montagekast', direction='i')  # i = direction: incoming

        self._type = OTLAttribuut(field=KlVoorschakelapparaatType,
                                  naam='type',
                                  label='type',
                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Voorschakelapparaat.type',
                                  definition='Type van het voorschakelapparaat.',
                                  owner=self)

    @property
    def type(self) -> str:
        """Type van het voorschakelapparaat."""
        return self._type.get_waarde()

    @type.setter
    def type(self, value):
        self._type.set_waarde(value, owner=self)



# coding=utf-8
from typing import List
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.ImplementatieElement.AIMNaamObject import AIMNaamObject
from ...Datatypes.DtcCompacteBatterij import DtcCompacteBatterij, DtcCompacteBatterijWaarden
from ...Datatypes.KlIoTSensorMerk import KlIoTSensorMerk
from ...Datatypes.KlIoTSensorModelnaam import KlIoTSensorModelnaam
from ...Datatypes.KlIoTSensorParameter import KlIoTSensorParameter
from ...Datatypes.KlIoTSensorVerbindingstype import KlIoTSensorVerbindingstype
from otlmow_model.OtlmowModel.BaseClasses.StringField import StringField
from otlmow_model.OtlmowModel.GeometrieTypes.PuntGeometrie import PuntGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class IoTSensor(AIMNaamObject, PuntGeometrie):
    """Een sensor die veranderingen in de omgeving registreert en erop reageert."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IoTSensor'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Draagconstructie', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#MeetstationAbstract', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Meetstation', direction='o', deprecated='2.14.0')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Sturing', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#SoftwareToegang', direction='u')  # u = unidirectional

        self._batterij = OTLAttribuut(field=DtcCompacteBatterij,
                                      naam='batterij',
                                      label='batterij',
                                      objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IoTSensor.batterij',
                                      definition='Bevat de informatie van de inwendige compacte batterij.',
                                      owner=self)

        self._gemetenParameters = OTLAttribuut(field=KlIoTSensorParameter,
                                               naam='gemetenParameters',
                                               label='gemeten parameters',
                                               objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IoTSensor.gemetenParameters',
                                               kardinaliteit_max='*',
                                               definition='De mogelijke parameters die kunnen gemeten worden door de IoT-sensor.',
                                               owner=self)

        self._merk = OTLAttribuut(field=KlIoTSensorMerk,
                                  naam='merk',
                                  label='merk',
                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IoTSensor.merk',
                                  definition='Het merk van een IoT-sensor.',
                                  owner=self)

        self._modelnaam = OTLAttribuut(field=KlIoTSensorModelnaam,
                                       naam='modelnaam',
                                       label='modelnaam',
                                       objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IoTSensor.modelnaam',
                                       definition='De modelnaam van een IoT-sensor.',
                                       owner=self)

        self._serienummer = OTLAttribuut(field=StringField,
                                         naam='serienummer',
                                         label='serienummer',
                                         objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IoTSensor.serienummer',
                                         definition='Het unieke nummer waarmee het toestel door de fabrikant geïdentificeerd is.',
                                         owner=self)

        self._typeVerbinding = OTLAttribuut(field=KlIoTSensorVerbindingstype,
                                            naam='typeVerbinding',
                                            label='type verbinding',
                                            objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IoTSensor.typeVerbinding',
                                            definition='De manier van de communicatieverbinding van de IoT-sensor.',
                                            owner=self)

    @property
    def batterij(self) -> DtcCompacteBatterijWaarden:
        """Bevat de informatie van de inwendige compacte batterij."""
        return self._batterij.get_waarde()

    @batterij.setter
    def batterij(self, value):
        self._batterij.set_waarde(value, owner=self)

    @property
    def gemetenParameters(self) -> List[str]:
        """De mogelijke parameters die kunnen gemeten worden door de IoT-sensor."""
        return self._gemetenParameters.get_waarde()

    @gemetenParameters.setter
    def gemetenParameters(self, value):
        self._gemetenParameters.set_waarde(value, owner=self)

    @property
    def merk(self) -> str:
        """Het merk van een IoT-sensor."""
        return self._merk.get_waarde()

    @merk.setter
    def merk(self, value):
        self._merk.set_waarde(value, owner=self)

    @property
    def modelnaam(self) -> str:
        """De modelnaam van een IoT-sensor."""
        return self._modelnaam.get_waarde()

    @modelnaam.setter
    def modelnaam(self, value):
        self._modelnaam.set_waarde(value, owner=self)

    @property
    def serienummer(self) -> str:
        """Het unieke nummer waarmee het toestel door de fabrikant geïdentificeerd is."""
        return self._serienummer.get_waarde()

    @serienummer.setter
    def serienummer(self, value):
        self._serienummer.set_waarde(value, owner=self)

    @property
    def typeVerbinding(self) -> str:
        """De manier van de communicatieverbinding van de IoT-sensor."""
        return self._typeVerbinding.get_waarde()

    @typeVerbinding.setter
    def typeVerbinding(self, value):
        self._typeVerbinding.set_waarde(value, owner=self)



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.Abstracten.SerienummerObject import SerienummerObject
from ...Classes.ImplementatieElement.NaampadObject import NaampadObject
from otlmow_model.OtlmowModel.BaseClasses.BooleanField import BooleanField
from ...Datatypes.DtcDocument import DtcDocument, DtcDocumentWaarden
from ...Datatypes.KlAlgMateriaal import KlAlgMateriaal
from ...Datatypes.KlZuilTGCMerk import KlZuilTGCMerk
from ...Datatypes.KlZuilTGCModelnaam import KlZuilTGCModelnaam
from otlmow_model.OtlmowModel.GeometrieTypes.PuntGeometrie import PuntGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class ZuilTGC(SerienummerObject, NaampadObject, PuntGeometrie):
    """Een zuil om badgelezers en videofoonposten op te plaatsen. Deze bevinden zich typisch aan slagbomen of automatische poorten."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#ZuilTGC'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#ANPRCamera', direction='u', deprecated='2.9.0')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Badgelezer', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Camera', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IntercomToestel', direction='u')  # u = unidirectional

        self._isGeschiktVoorPersonenvoertuigen = OTLAttribuut(field=BooleanField,
                                                              naam='isGeschiktVoorPersonenvoertuigen',
                                                              label='is geschikt voor personenvoertuigen',
                                                              objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#ZuilTGC.isGeschiktVoorPersonenvoertuigen',
                                                              definition='Geeft aan of de zuil een hoogte heeft die het geschikt maakt om te worden benut door chauffeurs van personenvoertuigen. Doorgaans is de zuil dan ook geschikt voor voetgangers.',
                                                              owner=self)

        self._isGeschiktVoorVrachtwagens = OTLAttribuut(field=BooleanField,
                                                        naam='isGeschiktVoorVrachtwagens',
                                                        label='is geschikt voor vrachtwagens',
                                                        objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#ZuilTGC.isGeschiktVoorVrachtwagens',
                                                        definition='Geeft aan of de zuil een hoogte heeft die het geschikt maakt om te worden benut door chauffeurs van vrachtwagens.',
                                                        owner=self)

        self._materiaal = OTLAttribuut(field=KlAlgMateriaal,
                                       naam='materiaal',
                                       label='materiaal',
                                       objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#ZuilTGC.materiaal',
                                       definition='Het materiaal waaruit de toegangscontrolezuil (voornamelijk) gemaakt is.',
                                       owner=self)

        self._merk = OTLAttribuut(field=KlZuilTGCMerk,
                                  naam='merk',
                                  label='merk',
                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#ZuilTGC.merk',
                                  definition='Het merk van de zuil.',
                                  owner=self)

        self._modelnaam = OTLAttribuut(field=KlZuilTGCModelnaam,
                                       naam='modelnaam',
                                       label='modelnaam',
                                       objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#ZuilTGC.modelnaam',
                                       definition='De modelnaam van de zuil.',
                                       owner=self)

        self._technischeFiche = OTLAttribuut(field=DtcDocument,
                                             naam='technischeFiche',
                                             label='technische fiche',
                                             objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#ZuilTGC.technischeFiche',
                                             definition='Documentatie met bijkomende (technische) details van de toegangscontrolezuil.',
                                             owner=self)

    @property
    def isGeschiktVoorPersonenvoertuigen(self) -> bool:
        """Geeft aan of de zuil een hoogte heeft die het geschikt maakt om te worden benut door chauffeurs van personenvoertuigen. Doorgaans is de zuil dan ook geschikt voor voetgangers."""
        return self._isGeschiktVoorPersonenvoertuigen.get_waarde()

    @isGeschiktVoorPersonenvoertuigen.setter
    def isGeschiktVoorPersonenvoertuigen(self, value):
        self._isGeschiktVoorPersonenvoertuigen.set_waarde(value, owner=self)

    @property
    def isGeschiktVoorVrachtwagens(self) -> bool:
        """Geeft aan of de zuil een hoogte heeft die het geschikt maakt om te worden benut door chauffeurs van vrachtwagens."""
        return self._isGeschiktVoorVrachtwagens.get_waarde()

    @isGeschiktVoorVrachtwagens.setter
    def isGeschiktVoorVrachtwagens(self, value):
        self._isGeschiktVoorVrachtwagens.set_waarde(value, owner=self)

    @property
    def materiaal(self) -> str:
        """Het materiaal waaruit de toegangscontrolezuil (voornamelijk) gemaakt is."""
        return self._materiaal.get_waarde()

    @materiaal.setter
    def materiaal(self, value):
        self._materiaal.set_waarde(value, owner=self)

    @property
    def merk(self) -> str:
        """Het merk van de zuil."""
        return self._merk.get_waarde()

    @merk.setter
    def merk(self, value):
        self._merk.set_waarde(value, owner=self)

    @property
    def modelnaam(self) -> str:
        """De modelnaam van de zuil."""
        return self._modelnaam.get_waarde()

    @modelnaam.setter
    def modelnaam(self, value):
        self._modelnaam.set_waarde(value, owner=self)

    @property
    def technischeFiche(self) -> DtcDocumentWaarden:
        """Documentatie met bijkomende (technische) details van de toegangscontrolezuil."""
        return self._technischeFiche.get_waarde()

    @technischeFiche.setter
    def technischeFiche(self, value):
        self._technischeFiche.set_waarde(value, owner=self)



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.Abstracten.TechnischDocument import TechnischDocument
from ...Classes.ImplementatieElement.AIMNaamObject import AIMNaamObject
from ...Datatypes.KlMateriaalAnker import KlMateriaalAnker
from ...Datatypes.KlMateriaalStalenKlemblok import KlMateriaalStalenKlemblok
from ...Datatypes.KlTypeAnker import KlTypeAnker
from otlmow_model.OtlmowModel.BaseClasses.NonNegIntegerField import NonNegIntegerField
from otlmow_model.OtlmowModel.GeometrieTypes.PuntGeometrie import PuntGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class Klemsysteem(TechnischDocument, AIMNaamObject, PuntGeometrie):
    """Stalen klemblokken en ankerstaven die de balg inklemmen tegen de vloer van het stuwhoofd en daarbij de positie fixeren."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Klemsysteem'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Balgmechanisme', direction='o')  # o = direction: outgoing

        self._hoeveelheidAnkers = OTLAttribuut(field=NonNegIntegerField,
                                               naam='hoeveelheidAnkers',
                                               label='hoeveelheid ankers',
                                               objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Klemsysteem.hoeveelheidAnkers',
                                               definition='Geeft het aantal ankers aan.',
                                               owner=self)

        self._materiaalAnker = OTLAttribuut(field=KlMateriaalAnker,
                                            naam='materiaalAnker',
                                            label='materiaal anker',
                                            objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Klemsysteem.materiaalAnker',
                                            definition='Geeft de mogelijke opties van materialen van de anker aan.',
                                            owner=self)

        self._materiaalStalenKlemblok = OTLAttribuut(field=KlMateriaalStalenKlemblok,
                                                     naam='materiaalStalenKlemblok',
                                                     label='materiaal stalen blok',
                                                     objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Klemsysteem.materiaalStalenKlemblok',
                                                     definition='Geeft de mogelijke opties van materiaal van het stalen blok aan.',
                                                     owner=self)

        self._typeAnker = OTLAttribuut(field=KlTypeAnker,
                                       naam='typeAnker',
                                       label='type anker',
                                       objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Klemsysteem.typeAnker',
                                       definition='Geeft de mogelijke opties van type anker aan.',
                                       owner=self)

    @property
    def hoeveelheidAnkers(self) -> int:
        """Geeft het aantal ankers aan."""
        return self._hoeveelheidAnkers.get_waarde()

    @hoeveelheidAnkers.setter
    def hoeveelheidAnkers(self, value):
        self._hoeveelheidAnkers.set_waarde(value, owner=self)

    @property
    def materiaalAnker(self) -> str:
        """Geeft de mogelijke opties van materialen van de anker aan."""
        return self._materiaalAnker.get_waarde()

    @materiaalAnker.setter
    def materiaalAnker(self, value):
        self._materiaalAnker.set_waarde(value, owner=self)

    @property
    def materiaalStalenKlemblok(self) -> str:
        """Geeft de mogelijke opties van materiaal van het stalen blok aan."""
        return self._materiaalStalenKlemblok.get_waarde()

    @materiaalStalenKlemblok.setter
    def materiaalStalenKlemblok(self, value):
        self._materiaalStalenKlemblok.set_waarde(value, owner=self)

    @property
    def typeAnker(self) -> str:
        """Geeft de mogelijke opties van type anker aan."""
        return self._typeAnker.get_waarde()

    @typeAnker.setter
    def typeAnker(self, value):
        self._typeAnker.set_waarde(value, owner=self)



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.Abstracten.TechnischDocument import TechnischDocument
from ...Classes.ImplementatieElement.AIMNaamObject import AIMNaamObject
from ...Datatypes.KlMateriaalBusEnRing import KlMateriaalBusEnRing
from otlmow_model.OtlmowModel.GeometrieTypes.PuntGeometrie import PuntGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class RingMechanisch(TechnischDocument, AIMNaamObject, PuntGeometrie):
    """Een cirkelvormig, meestal metalen object, dat wordt gebruikt als verbindings-, afdichtings- of ondersteuningselement in mechanische constructies. De hoogte is kleiner dan of gelijk aan de wanddikte."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#RingMechanisch'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Rolgeleiding', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Scharnierpunt', direction='o')  # o = direction: outgoing

        self._materiaalRing = OTLAttribuut(field=KlMateriaalBusEnRing,
                                           naam='materiaalRing',
                                           label='materiaal ring',
                                           objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#RingMechanisch.materiaalRing',
                                           definition='Het materiaal waaruit de ring is vervaardigd.',
                                           owner=self)

    @property
    def materiaalRing(self) -> str:
        """Het materiaal waaruit de ring is vervaardigd."""
        return self._materiaalRing.get_waarde()

    @materiaalRing.setter
    def materiaalRing(self, value):
        self._materiaalRing.set_waarde(value, owner=self)



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from abc import abstractmethod
from ...Classes.ImplementatieElement.AIMNaamObject import AIMNaamObject
from ...Datatypes.KlKabelFabrikant import KlKabelFabrikant
from ...Datatypes.KlKabelLeidingBescherming import KlKabelLeidingBescherming
from ...Datatypes.KlKabelmantelKleur import KlKabelmantelKleur
from ...Datatypes.KwantWrdInKilogramPerMeter import KwantWrdInKilogramPerMeter, KwantWrdInKilogramPerMeterWaarden
from ...Datatypes.KwantWrdInMillimeter import KwantWrdInMillimeter, KwantWrdInMillimeterWaarden
from otlmow_model.OtlmowModel.GeometrieTypes.LijnGeometrie import LijnGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class Kabel(AIMNaamObject, LijnGeometrie):
    """Abstracte voor attributen en relaties van allerlei types kabels."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Kabel'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    @abstractmethod
    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Doorverbinddoos', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Kabelmof', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HeeftAanvullendeGeometrie', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Onderdoorboring', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HeeftAanvullendeGeometrie', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Onderwaterkruising', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#CameraGroep', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#DynBordGroep', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#MIVModule', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Meteostation', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Slagboom', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Camera', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#DynBordRSS', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#DynBordVMS', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Omhult', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#OmhullendeInrichting', direction='i')  # i = direction: incoming

        self._buitenmantelDiameter = OTLAttribuut(field=KwantWrdInMillimeter,
                                                  naam='buitenmantelDiameter',
                                                  label='buitenmantel diameter',
                                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Kabel.buitenmantelDiameter',
                                                  definition='De buitenste afmeting van de kabel in millimeter.',
                                                  owner=self)

        self._buitenmantelKleur = OTLAttribuut(field=KlKabelmantelKleur,
                                               naam='buitenmantelKleur',
                                               label='buitenmantel kleur',
                                               objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Kabel.buitenmantelKleur',
                                               definition='De hoofdkleur(en) die voor een waarnemer onmiddellijk zichtbaar is (zijn) zonder de kabel te ontmantelen, de kleur van de markeringen op die buitenste mantel buiten beschouwing gelaten.',
                                               owner=self)

        self._fabrikant = OTLAttribuut(field=KlKabelFabrikant,
                                       naam='fabrikant',
                                       label='fabrikant',
                                       objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Kabel.fabrikant',
                                       definition='De naam van de producent van de kabel.',
                                       owner=self)

        self._gewicht = OTLAttribuut(field=KwantWrdInKilogramPerMeter,
                                     naam='gewicht',
                                     label='gewicht',
                                     objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Kabel.gewicht',
                                     definition='Het gewicht van de kabel per meter.',
                                     owner=self)

        self._typeBescherming = OTLAttribuut(field=KlKabelLeidingBescherming,
                                             naam='typeBescherming',
                                             label='type bescherming',
                                             objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Kabel.typeBescherming',
                                             definition='Geeft aan of en hoe de leiding bijkomend mechanisch beschermd is.',
                                             owner=self)

    @property
    def buitenmantelDiameter(self) -> KwantWrdInMillimeterWaarden:
        """De buitenste afmeting van de kabel in millimeter."""
        return self._buitenmantelDiameter.get_waarde()

    @buitenmantelDiameter.setter
    def buitenmantelDiameter(self, value):
        self._buitenmantelDiameter.set_waarde(value, owner=self)

    @property
    def buitenmantelKleur(self) -> str:
        """De hoofdkleur(en) die voor een waarnemer onmiddellijk zichtbaar is (zijn) zonder de kabel te ontmantelen, de kleur van de markeringen op die buitenste mantel buiten beschouwing gelaten."""
        return self._buitenmantelKleur.get_waarde()

    @buitenmantelKleur.setter
    def buitenmantelKleur(self, value):
        self._buitenmantelKleur.set_waarde(value, owner=self)

    @property
    def fabrikant(self) -> str:
        """De naam van de producent van de kabel."""
        return self._fabrikant.get_waarde()

    @fabrikant.setter
    def fabrikant(self, value):
        self._fabrikant.set_waarde(value, owner=self)

    @property
    def gewicht(self) -> KwantWrdInKilogramPerMeterWaarden:
        """Het gewicht van de kabel per meter."""
        return self._gewicht.get_waarde()

    @gewicht.setter
    def gewicht(self, value):
        self._gewicht.set_waarde(value, owner=self)

    @property
    def typeBescherming(self) -> str:
        """Geeft aan of en hoe de leiding bijkomend mechanisch beschermd is."""
        return self._typeBescherming.get_waarde()

    @typeBescherming.setter
    def typeBescherming(self, value):
        self._typeBescherming.set_waarde(value, owner=self)



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.ImplementatieElement.AIMNaamObject import AIMNaamObject
from otlmow_model.OtlmowModel.BaseClasses.BooleanField import BooleanField
from otlmow_model.OtlmowModel.BaseClasses.StringField import StringField
from otlmow_model.OtlmowModel.GeometrieTypes.PuntGeometrie import PuntGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class Toegangsvoorziening(AIMNaamObject, PuntGeometrie):
    """Een toegangsschouw of ruimte dat is ontworpen om personen in staat te stellen componenten van de sluis of stuw te bereiken, te inspecteren en eraan te werken."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Toegangsvoorziening'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Ballastcompartiment', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Ladder', direction='u')  # u = unidirectional

        self._afmetingen = OTLAttribuut(field=StringField,
                                        naam='afmetingen',
                                        label='afmetingen toegangsvoorziening',
                                        objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Toegangsvoorziening.afmetingen',
                                        definition='De afmetingen van de toegangsschouw en ruimte, uitgedrukt in centimeter.',
                                        owner=self)

        self._isVeiligheidsvoorzieningAanwezig = OTLAttribuut(field=BooleanField,
                                                              naam='isVeiligheidsvoorzieningAanwezig',
                                                              label='is veiligheidsvoorziening aanwezig',
                                                              objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Toegangsvoorziening.isVeiligheidsvoorzieningAanwezig',
                                                              definition='Geeft aan of er een veiligheidsvoorziening aanwezig is',
                                                              owner=self)

    @property
    def afmetingen(self) -> str:
        """De afmetingen van de toegangsschouw en ruimte, uitgedrukt in centimeter."""
        return self._afmetingen.get_waarde()

    @afmetingen.setter
    def afmetingen(self, value):
        self._afmetingen.set_waarde(value, owner=self)

    @property
    def isVeiligheidsvoorzieningAanwezig(self) -> bool:
        """Geeft aan of er een veiligheidsvoorziening aanwezig is"""
        return self._isVeiligheidsvoorzieningAanwezig.get_waarde()

    @isVeiligheidsvoorzieningAanwezig.setter
    def isVeiligheidsvoorzieningAanwezig(self, value):
        self._isVeiligheidsvoorzieningAanwezig.set_waarde(value, owner=self)



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.ImplementatieElement.AIMObject import AIMObject
from ...Datatypes.KlBouwputType import KlBouwputType
from ...Datatypes.KwantWrdInMeter import KwantWrdInMeter, KwantWrdInMeterWaarden
from otlmow_model.OtlmowModel.GeometrieTypes.VlakGeometrie import VlakGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class Bouwput(AIMObject, VlakGeometrie):
    """De ontgraving die nodig is voor het maken van een ondergrondse constructie zoals bv. een put of putten."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bouwput'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondkeringen', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Laag', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#PutRelatie', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Damwand', direction='u', deprecated='2.8.0')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#KringsBerliner', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Rioleringsstelsel', direction='o')  # o = direction: outgoing

        self._putdiepte = OTLAttribuut(field=KwantWrdInMeter,
                                       naam='putdiepte',
                                       label='putdiepte',
                                       objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bouwput.putdiepte',
                                       definition='Diepte tussen het maaiveld en onderkant bouwput in meter.',
                                       owner=self)

        self._type = OTLAttribuut(field=KlBouwputType,
                                  naam='type',
                                  label='type',
                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bouwput.type',
                                  definition='Het type van bouwput.',
                                  owner=self)

    @property
    def putdiepte(self) -> KwantWrdInMeterWaarden:
        """Diepte tussen het maaiveld en onderkant bouwput in meter."""
        return self._putdiepte.get_waarde()

    @putdiepte.setter
    def putdiepte(self, value):
        self._putdiepte.set_waarde(value, owner=self)

    @property
    def type(self) -> str:
        """Het type van bouwput."""
        return self._type.get_waarde()

    @type.setter
    def type(self, value):
        self._type.set_waarde(value, owner=self)



import json
from collections.abc import Iterable
from pathlib import Path

global_relation_dict_by_model: dict = {}
global_class_dict_by_model: dict = {}

MODEL_ROOT_PATH = Path(__file__).parent.parent.parent


def get_hardcoded_relation_dict(model_directory: Path = None) -> dict:
    global global_relation_dict_by_model
    model_dir_str = str(model_directory)
    if model_dir_str in global_relation_dict_by_model:
        return global_relation_dict_by_model[model_dir_str]

    if model_directory is None:
        model_directory = MODEL_ROOT_PATH
        model_dir_str = str(model_directory)

    generated_info_path = model_directory / 'OtlmowModel' / 'generated_info.json'
    if not generated_info_path.exists():
        raise FileNotFoundError(f"Generated info file not found at {generated_info_path}")

    with open(generated_info_path) as f:
        generated_info_dict = json.load(f)
    global_relation_dict_by_model[model_dir_str] = generated_info_dict['relations']

    return global_relation_dict_by_model[model_dir_str]


def get_hardcoded_class_dict(model_directory: Path = None) -> dict:
    global global_class_dict_by_model
    model_dir_str = str(model_directory)
    if model_dir_str in global_class_dict_by_model:
        return global_class_dict_by_model[model_dir_str]

    if model_directory is None:
        model_directory = MODEL_ROOT_PATH
        model_dir_str = str(model_directory)

    generated_info_path = model_directory / 'OtlmowModel' / 'generated_info.json'
    if not generated_info_path.exists():
        raise FileNotFoundError(f"Generated info file not found at {generated_info_path}")

    with open(generated_info_path) as f:
        generated_info_dict = json.load(f)
    global_class_dict_by_model[model_dir_str] = generated_info_dict['classes']

    return global_class_dict_by_model[model_dir_str]

def get_concrete_subclasses_from_class_dict(base_uri: str, model_directory: Path = None) -> Iterable[str]:
    class_dict = get_hardcoded_class_dict(model_directory)
    for subclass in class_dict[base_uri]['direct_subclasses']:
        if not class_dict[subclass]['abstract']:
            yield subclass
        yield from get_concrete_subclasses_from_class_dict(subclass, model_directory)


# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.KeuzelijstField import KeuzelijstField
from otlmow_model.OtlmowModel.BaseClasses.KeuzelijstWaarde import KeuzelijstWaarde


# Generated with OTLEnumerationCreator. To modify: extend, do not edit
class KlSluisCategorie(KeuzelijstField):
    """De mogelijke categorieën van een sluis."""
    naam = 'KlSluisCategorie'
    label = 'Sluiscategorie'
    objectUri = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#KlSluisCategorie'
    definition = 'De mogelijke categorieën van een sluis.'
    status = 'ingebruik'
    codelist = 'https://wegenenverkeer.data.vlaanderen.be/id/conceptscheme/KlSluisCategorie'
    options = {
        'binnenvaartsluis': KeuzelijstWaarde(invulwaarde='binnenvaartsluis',
                                             label='binnenvaartsluis',
                                             status='ingebruik',
                                             objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlSluisCategorie/binnenvaartsluis'),
        'getijdesluis': KeuzelijstWaarde(invulwaarde='getijdesluis',
                                         label='getijdesluis',
                                         status='ingebruik',
                                         objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlSluisCategorie/getijdesluis'),
        'stuwsluis': KeuzelijstWaarde(invulwaarde='stuwsluis',
                                      label='stuwsluis',
                                      status='ingebruik',
                                      objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlSluisCategorie/stuwsluis'),
        'zeesluis': KeuzelijstWaarde(invulwaarde='zeesluis',
                                     label='zeesluis',
                                     status='ingebruik',
                                     objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlSluisCategorie/zeesluis')
    }

    @classmethod
    def create_dummy_data(cls):
        return cls.create_dummy_data_keuzelijst(cls.options)




# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.Abstracten.AxiaalDraagvermogen import AxiaalDraagvermogen
from ...Classes.Abstracten.Fundering import Fundering
from ...Datatypes.DtuDwarsafmetingen import DtuDwarsafmetingen, DtuDwarsafmetingenWaarden
from ...Datatypes.DtuHellingsSchoorhoek import DtuHellingsSchoorhoek, DtuHellingsSchoorhoekWaarden
from ...Datatypes.KwantWrdInMeter import KwantWrdInMeter, KwantWrdInMeterWaarden
from ...Datatypes.KwantWrdInMeterTAW import KwantWrdInMeterTAW, KwantWrdInMeterTAWWaarden
from otlmow_model.OtlmowModel.GeometrieTypes.PuntGeometrie import PuntGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class Funderingspaal(AxiaalDraagvermogen, Fundering, PuntGeometrie):
    """Diepfundering waarbij d.m.v. palen de belasting wordt afgedragen naar de diepe ondergrond. Enkel te gebruiken wanneer het type paal niet gekend is."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingspaal'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Behuizing', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#BekledingComponent', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondanker', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Hoppinzuil', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Inloopbehuizing', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#KabelgeleidingEnLeidingBevestiging', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Kast', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#NietWeggebondenDetectie', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Seinlantaarn', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#SteunStandaard', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Ventilatie', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Verkeersbord', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Verlichtingstoestel', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Waarschuwingslantaarn', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#ZenderOntvangerToegang', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Binnenverlichtingstoestel', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingszool', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Kokerruimte', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Brugdeel', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#DrogePompkelder', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Kelderlandhoofd', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Kelderpijler', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Koker', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Kokersectie', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Landhoofd', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Lokaal', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Paalmatras', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Pijler', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Pompkamer', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#LigtOp', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#ConstructieElement', direction='i')  # i = direction: incoming
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#LigtOp', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Laag', direction='i')  # i = direction: incoming
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#LigtOp', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#GewapendeGrond', direction='i')  # i = direction: incoming
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#SluitAanOp', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#ComplexeGeleiding', direction='i')  # i = direction: incoming

        self._afkappeil = OTLAttribuut(field=KwantWrdInMeterTAW,
                                       naam='afkappeil',
                                       label='afkap- of bovenpeil',
                                       objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingspaal.afkappeil',
                                       definition='De hoogte van het bovenvlak van de paal, na verwijderen van eventuele overlengte, exclusief uitstekende wapening. Berekend ten opzichte van gemiddeld laagwaterpeil te Oostende (TAWpeil). Afkappeil wordt enkel gebruikt voor in-situ betonpalen.',
                                       owner=self)

        self._dwarsafmetingen = OTLAttribuut(field=DtuDwarsafmetingen,
                                             naam='dwarsafmetingen',
                                             label='dwarsafmetingen',
                                             objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingspaal.dwarsafmetingen',
                                             definition='Dwarsdoorsnede van het element bv. lengte, breedte, diameter,...',
                                             owner=self)

        self._hellingsSchoorhoek = OTLAttribuut(field=DtuHellingsSchoorhoek,
                                                naam='hellingsSchoorhoek',
                                                label='hellings- of schoorhoek',
                                                objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingspaal.hellingsSchoorhoek',
                                                definition='De hoek die de paal maakt ten opzichte van de verticale, uitgedrukt in decimale graden of in 1 op x.',
                                                owner=self)

        self._paallengte = OTLAttribuut(field=KwantWrdInMeter,
                                        naam='paallengte',
                                        label='paallengte',
                                        objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingspaal.paallengte',
                                        definition='Afstand gemeten, in meter, volgens de as van de paal tussen het afkappeil en het aanzetpeil.',
                                        owner=self)

    @property
    def afkappeil(self) -> KwantWrdInMeterTAWWaarden:
        """De hoogte van het bovenvlak van de paal, na verwijderen van eventuele overlengte, exclusief uitstekende wapening. Berekend ten opzichte van gemiddeld laagwaterpeil te Oostende (TAWpeil). Afkappeil wordt enkel gebruikt voor in-situ betonpalen."""
        return self._afkappeil.get_waarde()

    @afkappeil.setter
    def afkappeil(self, value):
        self._afkappeil.set_waarde(value, owner=self)

    @property
    def dwarsafmetingen(self) -> DtuDwarsafmetingenWaarden:
        """Dwarsdoorsnede van het element bv. lengte, breedte, diameter,..."""
        return self._dwarsafmetingen.get_waarde()

    @dwarsafmetingen.setter
    def dwarsafmetingen(self, value):
        self._dwarsafmetingen.set_waarde(value, owner=self)

    @property
    def hellingsSchoorhoek(self) -> DtuHellingsSchoorhoekWaarden:
        """De hoek die de paal maakt ten opzichte van de verticale, uitgedrukt in decimale graden of in 1 op x."""
        return self._hellingsSchoorhoek.get_waarde()

    @hellingsSchoorhoek.setter
    def hellingsSchoorhoek(self, value):
        self._hellingsSchoorhoek.set_waarde(value, owner=self)

    @property
    def paallengte(self) -> KwantWrdInMeterWaarden:
        """Afstand gemeten, in meter, volgens de as van de paal tussen het afkappeil en het aanzetpeil."""
        return self._paallengte.get_waarde()

    @paallengte.setter
    def paallengte(self, value):
        self._paallengte.set_waarde(value, owner=self)



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.KeuzelijstField import KeuzelijstField
from otlmow_model.OtlmowModel.BaseClasses.KeuzelijstWaarde import KeuzelijstWaarde


# Generated with OTLEnumerationCreator. To modify: extend, do not edit
class KlSeinlantaarnModelnaam(KeuzelijstField):
    """Keuzelijst met modelnamen voor Seinlantaarn."""
    naam = 'KlSeinlantaarnModelnaam'
    label = 'seinlantaarn modelnaam'
    objectUri = 'https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#KlSeinlantaarnModelnaam'
    definition = 'Keuzelijst met modelnamen voor Seinlantaarn.'
    status = 'ingebruik'
    codelist = 'https://wegenenverkeer.data.vlaanderen.be/id/conceptscheme/KlSeinlantaarnModelnaam'
    options = {
        'alustar': KeuzelijstWaarde(invulwaarde='alustar',
                                    label='Alustar',
                                    status='ingebruik',
                                    definitie='Alustar',
                                    objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlSeinlantaarnModelnaam/alustar'),
        'ciway': KeuzelijstWaarde(invulwaarde='ciway',
                                  label='CIWAY',
                                  status='ingebruik',
                                  definitie='CIWAY',
                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlSeinlantaarnModelnaam/ciway'),
        'futurit': KeuzelijstWaarde(invulwaarde='futurit',
                                    label='Futurit',
                                    status='ingebruik',
                                    definitie='Futurit',
                                    objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlSeinlantaarnModelnaam/futurit')
    }

    @classmethod
    def create_dummy_data(cls):
        return cls.create_dummy_data_keuzelijst(cls.options)




# coding=utf-8
from typing import List
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from ...Classes.Abstracten.Buitenkast import Buitenkast
from otlmow_model.OtlmowModel.BaseClasses.BooleanField import BooleanField
from ...Datatypes.DtcDocument import DtcDocument, DtcDocumentWaarden
from ...Datatypes.KlWegkantkastType import KlWegkantkastType


# Generated with OTLClassCreator. To modify: extend, do not edit
class Wegkantkast(Buitenkast):
    """Behuizing in de vorm van een kast typisch gebruikt buiten,langs de kant van de weg."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Wegkantkast'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Laag', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Toegangselement', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#MIVModule', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#AIDModule', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Antenne', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Cabinecontroller', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Handbediening', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#IntercomToestel', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#LoopTerminationAndProtection', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#MIVVoedingsmodule', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#PTRegelaar', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Segmentcontroller', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#ThermoHygrometer', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Verkeersregelaar', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#WIMDatalogger', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#BinnenverlichtingGroep', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#CameraGroep', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#DynBordGroep', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Tunnelverlichting', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Z30Groep', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Camera', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#PunctueleVerlichtingsmast', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Sturing', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#LEDBord', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Voedt', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Illuminator', direction='o')  # o = direction: outgoing

        self._elektrischSchema = OTLAttribuut(field=DtcDocument,
                                              naam='elektrischSchema',
                                              label='elektrisch schema',
                                              objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Wegkantkast.elektrischSchema',
                                              usagenote='Bestanden van het type pdf,dwg of dxf.',
                                              kardinaliteit_max='*',
                                              definition='Elektrisch aansluitschema van de kast.',
                                              owner=self)

        self._heeftMaaibescherming = OTLAttribuut(field=BooleanField,
                                                  naam='heeftMaaibescherming',
                                                  label='heeft maaibescherming',
                                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Wegkantkast.heeftMaaibescherming',
                                                  definition='Geeft aan of de kast voorzien is van bescherming tegen schade bij het maaien van de omgeving rond de kast.',
                                                  owner=self)

        self._mplan = OTLAttribuut(field=DtcDocument,
                                   naam='mplan',
                                   label='m-plan',
                                   objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Wegkantkast.mplan',
                                   usagenote='Bestanden van het type dwg of pdf.',
                                   kardinaliteit_max='*',
                                   definition='Mechanisch plan van de volledige installatie. Er wordt één plan toegevoegd per installatie/techniek die op de kast is aangesloten.',
                                   owner=self)

        self._type = OTLAttribuut(field=KlWegkantkastType,
                                  naam='type',
                                  label='type',
                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Wegkantkast.type',
                                  definition='Type van de wegkantkast volgens de gangbare types.',
                                  owner=self)

    @property
    def elektrischSchema(self) -> List[DtcDocumentWaarden]:
        """Elektrisch aansluitschema van de kast."""
        return self._elektrischSchema.get_waarde()

    @elektrischSchema.setter
    def elektrischSchema(self, value):
        self._elektrischSchema.set_waarde(value, owner=self)

    @property
    def heeftMaaibescherming(self) -> bool:
        """Geeft aan of de kast voorzien is van bescherming tegen schade bij het maaien van de omgeving rond de kast."""
        return self._heeftMaaibescherming.get_waarde()

    @heeftMaaibescherming.setter
    def heeftMaaibescherming(self, value):
        self._heeftMaaibescherming.set_waarde(value, owner=self)

    @property
    def mplan(self) -> List[DtcDocumentWaarden]:
        """Mechanisch plan van de volledige installatie. Er wordt één plan toegevoegd per installatie/techniek die op de kast is aangesloten."""
        return self._mplan.get_waarde()

    @mplan.setter
    def mplan(self, value):
        self._mplan.set_waarde(value, owner=self)

    @property
    def type(self) -> str:
        """Type van de wegkantkast volgens de gangbare types."""
        return self._type.get_waarde()

    @type.setter
    def type(self, value):
        self._type.set_waarde(value, owner=self)



# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.KeuzelijstField import KeuzelijstField
from otlmow_model.OtlmowModel.BaseClasses.KeuzelijstWaarde import KeuzelijstWaarde


# Generated with OTLEnumerationCreator. To modify: extend, do not edit
class KlLEACTypeEindstuk(KeuzelijstField):
    """De verschillende types eindstukken."""
    naam = 'KlLEACTypeEindstuk'
    label = 'Type eindstuk'
    objectUri = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#KlLEACTypeEindstuk'
    definition = 'De verschillende types eindstukken.'
    status = 'ingebruik'
    codelist = 'https://wegenenverkeer.data.vlaanderen.be/id/conceptscheme/KlLEACTypeEindstuk'
    options = {
        'naar-beneden-afgebogen': KeuzelijstWaarde(invulwaarde='naar-beneden-afgebogen',
                                                   label='naar beneden afgebogen',
                                                   status='ingebruik',
                                                   definitie='naar beneden afgebogen',
                                                   objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlLEACTypeEindstuk/naar-beneden-afgebogen'),
        'schelp': KeuzelijstWaarde(invulwaarde='schelp',
                                   label='schelp',
                                   status='ingebruik',
                                   definitie='schelp',
                                   objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlLEACTypeEindstuk/schelp'),
        'uitgebogen': KeuzelijstWaarde(invulwaarde='uitgebogen',
                                       label='uitgebogen',
                                       status='ingebruik',
                                       definitie='uitgebogen',
                                       objectUri='https://wegenenverkeer.data.vlaanderen.be/id/concept/KlLEACTypeEindstuk/uitgebogen')
    }

    @classmethod
    def create_dummy_data(cls):
        return cls.create_dummy_data_keuzelijst(cls.options)




# coding=utf-8
from otlmow_model.OtlmowModel.BaseClasses.OTLObject import OTLAttribuut
from abc import abstractmethod
from ...Classes.Abstracten.AxiaalDraagvermogen import AxiaalDraagvermogen
from ...Classes.Abstracten.Grondkeringen import Grondkeringen
from otlmow_model.OtlmowModel.BaseClasses.BooleanField import BooleanField
from ...Datatypes.DtcConstructiestaalspecificaties import DtcConstructiestaalspecificaties, DtcConstructiestaalspecificatiesWaarden
from ...Datatypes.DtcDocument import DtcDocument, DtcDocumentWaarden
from ...Datatypes.KlTypeGrondanker import KlTypeGrondanker
from ...Datatypes.KwantWrdInVierkanteMillimeter import KwantWrdInVierkanteMillimeter, KwantWrdInVierkanteMillimeterWaarden
from otlmow_model.OtlmowModel.GeometrieTypes.PuntGeometrie import PuntGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class Grondanker(AxiaalDraagvermogen, Grondkeringen, PuntGeometrie):
    """Abstracte voor het bundelen van de verschillende grondankers."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondanker'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    @abstractmethod
    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#AfmetingenDiepCBWand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#BalkGK', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Betonfundering', direction='u', deprecated='2.0.0')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#DamplankAbstracte', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Fundering', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondanker', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondkeringen', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#KlassiekeFundering', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Berlinerwand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Combiwand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Damwand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Funderingswand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#GewapendeGrond', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Gewichtsmuur', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Lmuur', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#MVPaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#SecansTangenspalenwand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Soilmixwand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#StalenFunderingsprofiel', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#VernageldeWand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Baret', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#BetonnenBoorpaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#BetonnenCaison', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#BetonnenDamplank', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#BetonnenHeipaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#BetonnenSchroefpaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#CFAPaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#CementBentonietwand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Diepwand', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#FunderingOpStaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingsmassief', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingspaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingsplaat', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingsput', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Funderingszool', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Grindkern', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#GrondankerVerdeelplaat', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Groutpaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoutenDamplank', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoutenHeipaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Injectiepaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#KunststoffenDamplank', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Micropaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Sleufbekisting', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#SoilmixwandElement', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#StalenBuispaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#StalenDamplank', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#StalenSchroefpaal', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Verankeringsmassief', direction='u')  # u = unidirectional
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Kokerruimte', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Combiwand', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Damwand', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#DrogePompkelder', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Funderingswand', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Koker', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Kokersectie', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Lmuur', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Lokaal', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#Pompkamer', direction='o')  # o = direction: outgoing
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#HoortBij', target='https://wegenenverkeer.data.vlaanderen.be/ns/installatie#VernageldeWand', direction='o')  # o = direction: outgoing

        self._aanvaardingsdocument = OTLAttribuut(field=DtcDocument,
                                                  naam='aanvaardingsdocument',
                                                  label='aanvaardingsdocument',
                                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondanker.aanvaardingsdocument',
                                                  definition='Document dat aangeeft of het grondanker voor de aanvaardingsproef geslaagd is.',
                                                  owner=self)

        self._heeftBeschermingskap = OTLAttribuut(field=BooleanField,
                                                  naam='heeftBeschermingskap',
                                                  label='heeft beschermingskap',
                                                  objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondanker.heeftBeschermingskap',
                                                  definition='Geeft aan of het grondanker een beschermingskap heeft, al dan niet.',
                                                  owner=self)

        self._isActiefanker = OTLAttribuut(field=BooleanField,
                                           naam='isActiefanker',
                                           label='is actief anker',
                                           objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondanker.isActiefanker',
                                           definition='Geeft aan of het anker actief (voorgespannen) of passief (niet voorgespannen) is.',
                                           owner=self)

        self._staalsectie = OTLAttribuut(field=KwantWrdInVierkanteMillimeter,
                                         naam='staalsectie',
                                         label='staalsectie',
                                         objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondanker.staalsectie',
                                         definition='Totale oppervlakte van staalsectie, uitgedrukte in vierkant millimeter.',
                                         owner=self)

        self._staalspecificaties = OTLAttribuut(field=DtcConstructiestaalspecificaties,
                                                naam='staalspecificaties',
                                                label='staalspecificaties',
                                                objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondanker.staalspecificaties',
                                                definition='Type constructiestaal dat wordt gebruikt.',
                                                owner=self)

        self._typeGrondanker = OTLAttribuut(field=KlTypeGrondanker,
                                            naam='typeGrondanker',
                                            label='type grondanker',
                                            objectUri='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Grondanker.typeGrondanker',
                                            definition='Keuzelijst om het onderscheid te maken tussen mono, duo of gestakkelde ankers.',
                                            owner=self)

    @property
    def aanvaardingsdocument(self) -> DtcDocumentWaarden:
        """Document dat aangeeft of het grondanker voor de aanvaardingsproef geslaagd is."""
        return self._aanvaardingsdocument.get_waarde()

    @aanvaardingsdocument.setter
    def aanvaardingsdocument(self, value):
        self._aanvaardingsdocument.set_waarde(value, owner=self)

    @property
    def heeftBeschermingskap(self) -> bool:
        """Geeft aan of het grondanker een beschermingskap heeft, al dan niet."""
        return self._heeftBeschermingskap.get_waarde()

    @heeftBeschermingskap.setter
    def heeftBeschermingskap(self, value):
        self._heeftBeschermingskap.set_waarde(value, owner=self)

    @property
    def isActiefanker(self) -> bool:
        """Geeft aan of het anker actief (voorgespannen) of passief (niet voorgespannen) is."""
        return self._isActiefanker.get_waarde()

    @isActiefanker.setter
    def isActiefanker(self, value):
        self._isActiefanker.set_waarde(value, owner=self)

    @property
    def staalsectie(self) -> KwantWrdInVierkanteMillimeterWaarden:
        """Totale oppervlakte van staalsectie, uitgedrukte in vierkant millimeter."""
        return self._staalsectie.get_waarde()

    @staalsectie.setter
    def staalsectie(self, value):
        self._staalsectie.set_waarde(value, owner=self)

    @property
    def staalspecificaties(self) -> DtcConstructiestaalspecificatiesWaarden:
        """Type constructiestaal dat wordt gebruikt."""
        return self._staalspecificaties.get_waarde()

    @staalspecificaties.setter
    def staalspecificaties(self, value):
        self._staalspecificaties.set_waarde(value, owner=self)

    @property
    def typeGrondanker(self) -> str:
        """Keuzelijst om het onderscheid te maken tussen mono, duo of gestakkelde ankers."""
        return self._typeGrondanker.get_waarde()

    @typeGrondanker.setter
    def typeGrondanker(self, value):
        self._typeGrondanker.set_waarde(value, owner=self)
